////////////////////////////////////////////////////////////////////////////////
/// @brief variant class for vectors
///
/// @file
///
/// DISCLAIMER
///
/// Copyright 2010-2011 triagens GmbH, Cologne, Germany
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///
///     http://www.apache.org/licenses/LICENSE-2.0
///
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// Copyright holder is triAGENS GmbH, Cologne, Germany
///
/// @author Dr. Frank Celler
/// @author Achim Brandt
/// @author Copyright 2008-2011, triAGENS GmbH, Cologne, Germany
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIAGENS_BASICS_VARIANT_VECTOR_H
#define TRIAGENS_BASICS_VARIANT_VECTOR_H 1

#include <Basics/VariantObject.h>

namespace triagens {
  namespace basics {

    ////////////////////////////////////////////////////////////////////////////////
    /// @ingroup Variants
    /// @brief variant class for vectors
    ////////////////////////////////////////////////////////////////////////////////

    class VariantVector : public VariantObject {
      public:

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief type of VariantObject
        ////////////////////////////////////////////////////////////////////////////////

        static ObjectType const TYPE = VARIANT_VECTOR;

      public:

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief constructs a new vector
        ////////////////////////////////////////////////////////////////////////////////

        VariantVector ();

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief constructs a new vector
        ////////////////////////////////////////////////////////////////////////////////

        explicit
        VariantVector (vector<string> const&);

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief constructs a new vector
        ////////////////////////////////////////////////////////////////////////////////

        ~VariantVector ();

      public:

        ////////////////////////////////////////////////////////////////////////////////
        /// {@inheritDoc}
        ////////////////////////////////////////////////////////////////////////////////

        ObjectType type () const {
          return TYPE;
        }

        ////////////////////////////////////////////////////////////////////////////////
        /// {@inheritDoc}
        ////////////////////////////////////////////////////////////////////////////////

        VariantObject* clone () const;

        ////////////////////////////////////////////////////////////////////////////////
        /// {@inheritDoc}
        ////////////////////////////////////////////////////////////////////////////////

        void print (StringBuffer&, size_t indent) const;

      public:

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief returns the values
        ////////////////////////////////////////////////////////////////////////////////

        vector<VariantObject*> const& getValues () const {
          return values;
        }

      public:

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief adds a value
        ////////////////////////////////////////////////////////////////////////////////

        void add (VariantObject* value);

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief adds many values
        ////////////////////////////////////////////////////////////////////////////////

        void add (vector<VariantObject*> const& v);

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief adds a string value
        ////////////////////////////////////////////////////////////////////////////////

        void add (string const& value);

        ////////////////////////////////////////////////////////////////////////////////
        /// @brief adds many string values
        ////////////////////////////////////////////////////////////////////////////////

        void add (vector<string> const& v);

      private:
        vector<VariantObject*> values;
    };
  }
}

#endif
